@@ -403,6 +403,13 @@ window_set_wmclass_class (GladeXML *xml, GtkWidget *w,
     g_free(wmname);
 }
 
+static void
+entry_set_invisible_char (GladeXML *xml, GtkWidget *w,
+			  const gchar *name, const gchar *value)
+{
+    gtk_entry_set_invisible_char (GTK_ENTRY (w), value [0]);
+}
+
 static void
 button_set_response_id (GladeXML *xml, GtkWidget *w,
 			const gchar *name, const gchar *value)
@@ -471,6 +478,43 @@ gtk_dialog_build_children(GladeXML *self, GtkWidget *w,
     g_list_free (children);
 }
 
+
+static void
+frame_build_children(GladeXML *self, GtkWidget *parent,
+		     GladeWidgetInfo *info)
+{
+    gint i, j;
+    enum {
+	FRAME_ITEM,
+	LABEL_ITEM
+    } type;
+
+    g_object_ref(G_OBJECT(parent));
+    for (i = 0; i < info->n_children; i++) {
+	GladeWidgetInfo *childinfo = info->children[i].child;
+	GtkWidget *child = glade_xml_build_widget(self, childinfo);
+
+	type = FRAME_ITEM;
+	/* there should really only be 2 children */
+	for (j = 0; j < info->children[i].n_properties; j++) {
+	    if (!strcmp (info->children[i].properties[j].name, "type")) {
+		const char *value = info->children[i].properties[j].value;
+
+		if (!strcmp (value, "label_item"))
+		    type = LABEL_ITEM;
+		break;
+	    }
+	}
+
+	if (type == LABEL_ITEM) {
+	    gtk_frame_set_label_widget (GTK_FRAME (parent), child);
+	} else {
+	    gtk_container_add (GTK_CONTAINER (parent), child);
+	}
+    }
+    g_object_unref(G_OBJECT(parent));
+}
+
 static void
 notebook_build_children(GladeXML *self, GtkWidget *parent,
 			GladeWidgetInfo *info)
@@ -548,7 +592,7 @@ option_menu_build_children (GladeXML *xml, GtkWidget *parent,
 	GtkWidget *child;
 	GladeWidgetInfo *childinfo = info->children[i].child;
 
-	if (strcmp(childinfo->class, "GtkMenu") != 0) {
+	if (strcmp(childinfo->classname, "GtkMenu") != 0) {
 	    g_warning("the child of the option menu '%s' was not a GtkMenu",
 		      info->name);
 	    continue;
@@ -587,7 +631,7 @@ clist_build_children(GladeXML *self, GtkWidget *parent,
 	childinfo = info->children[i].child;
 
 	/* treat GtkLabels specially */
-	if (!strcmp (childinfo->class, "GtkLabel")) {
+	if (!strcmp (childinfo->classname, "GtkLabel")) {
 	    int j;
 	    const char *label = NULL;
 
@@ -632,9 +676,9 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 
 	childinfo = &info->children[i];
 
-	if (!strcmp (childinfo->child->class, "toggle") ||
-	    !strcmp (childinfo->child->class, "radio") ||
-	    !strcmp (childinfo->child->class, "button")) {
+	if (!strcmp (childinfo->child->classname, "toggle") ||
+	    !strcmp (childinfo->child->classname, "radio") ||
+	    !strcmp (childinfo->child->classname, "button")) {
 	    const char *label = NULL, *stock = NULL, *group_name = NULL;
 	    char *icon = NULL;
 	    gboolean use_stock = FALSE, active = FALSE, new_group = FALSE;
@@ -709,14 +753,14 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 		gtk_toolbar_append_space (GTK_TOOLBAR (parent));
 
 	    /* FIXME: these should be translated */
-	    if (!strcmp (childinfo->child->class, "toggle")) {
+	    if (!strcmp (childinfo->child->classname, "toggle")) {
 		child = gtk_toolbar_append_element (
 		    GTK_TOOLBAR (parent),
 		    GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL,
 		    label, NULL, NULL, iconw, NULL, NULL);
 		gtk_toggle_button_set_active(
 		    GTK_TOGGLE_BUTTON (child), active);
-	    } else if (!strcmp (childinfo->child->class, "radio")) {
+	    } else if (!strcmp (childinfo->child->classname, "radio")) {
 		child = gtk_toolbar_append_element (
 		    GTK_TOOLBAR (parent),
 		    GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL,
@@ -985,6 +1029,7 @@ _glade_init_gtk_widgets(void)
     glade_register_custom_prop (GTK_TYPE_WINDOW, "wmclass_class", window_set_wmclass_class);
     glade_register_custom_prop (GTK_TYPE_LIST_ITEM, "label", list_item_set_label);
     glade_register_custom_prop (GTK_TYPE_BUTTON, "response_id", button_set_response_id);
+    glade_register_custom_prop (GTK_TYPE_ENTRY, "invisible_char", entry_set_invisible_char);
 
     glade_register_widget (GTK_TYPE_ACCEL_LABEL, glade_standard_build_widget,
 			   NULL, NULL);
@@ -993,7 +1038,7 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_ARROW, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_ASPECT_FRAME, glade_standard_build_widget,
-			   glade_standard_build_children, NULL);
+			   frame_build_children, NULL);
     glade_register_widget (GTK_TYPE_BUTTON, glade_standard_build_widget,
 			   glade_standard_build_children, NULL);
     glade_register_widget (GTK_TYPE_CALENDAR, glade_standard_build_widget,
@@ -1031,7 +1076,7 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_FONT_SELECTION_DIALOG, NULL,
 			   glade_standard_build_children, fontseldlg_find_internal_child);
     glade_register_widget (GTK_TYPE_FRAME, glade_standard_build_widget,
-			   glade_standard_build_children, NULL);
+			   frame_build_children, NULL);
     glade_register_widget (GTK_TYPE_GAMMA_CURVE, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_HANDLE_BOX, glade_standard_build_widget,
